/* An object containing a counter and a main method that starts 10 threads.
 * Each of these threads will then read the counter, increment the local copy,
 * sleep for 100 milliseconds. Then the thread prints HelloWorld and the 
 * threads value of the counter. With race conditions the same value may appear
 * multiple times, without race conditions, each value should only appear once. 
 */
object MUTS_Demo_TM1 {
  var counter = 0
  def main(args: Array[String]): Unit = {
    for(i <- 1 to 10)
      new DemoThread_TM1().start()
    
  }
}

/* A class extending Thread. This version uses annotations to mark methods as
 * transactional. Because the whole method is transactional, it has been 
 * necessary to  separate the run method into two methods in order to prevent
 * I/O appearing in the transaction. This restriction is overcome by the 
 * alternative syntax demonstrated in MUTS_DEmo_TM2. 
 */
import org.deuce.Atomic
class DemoThread_TM1 extends Thread {
  override def run():Unit = {
    val counter = incrementCounter
    println("Hello World " + counter)
  }
  
  @Atomic
  def incrementCounter():Int = {
    var counter = MUTS_Demo_TM1.counter
    counter += 1
    Thread.sleep(100)
    MUTS_Demo_TM1.counter = counter
    counter
  }
}