/* An object containing a counter and a main method that starts 10 threads.
 * Each of these threads will then read the counter, increment the local copy,
 * sleep for 100 milliseconds. Then the thread prints HelloWorld and the 
 * threads value of the counter. With race conditions the same value may appear
 * multiple times, without race conditions, each value should only appear once. 
 */
object MUTS_Demo_TM2 {
  var counter = 0
  def main(args: Array[String]): Unit = {
    for(i <- 1 to 10)
      new DemoThread_TM2().start()   
  }
}

/* A Method extending Thread. This version uses a library call taking a closure
 * to mark the transactional region. This allows finer grained control of the 
 * transaction so removing the need to refactor the methods.
 */
import eu.teraflux.uniman.transactions.TMLib._
class DemoThread_TM2 extends Thread {
  override def run():Unit = {
    var counter = 0
    
    atomic{ Unit => 
      counter = MUTS_Demo_noTM.counter
      counter += 1
      Thread.sleep(100)
      MUTS_Demo_noTM.counter = counter
    }
    
    println("Hello World " + counter)
  }
}